@@ -36,7 +36,9 @@ class Client:
 
     CALLBACK_THREADS_NUM = 5
 
-    def __init__(self, client_configuration, topics, client_type: ClientType, tls_enable=False):
+    def __init__(
+        self, client_configuration, topics, client_type: ClientType, tls_enable=False
+    ):
         if client_configuration is None:
             raise IllegalArgumentException("clientConfiguration should not be null.")
         self.__client_configuration = client_configuration
@@ -55,7 +57,9 @@ class Client:
         self.__sync_setting_scheduler_threading_event = None
         self.__clear_idle_rpc_channels_threading_event = None
         if topics is not None:
-            self.__topics = set(filter(lambda topic: Misc.is_valid_topic(topic), topics))
+            self.__topics = set(
+                filter(lambda topic: Misc.is_valid_topic(topic), topics)
+            )
         else:
             self.__topics = set()
         self.__callback_result_queue = Queue()
@@ -67,7 +71,9 @@ class Client:
     def startup(self):
         try:
             if self.__had_shutdown is True:
-                raise Exception(f"client:{self.__client_id} had shutdown, can't startup again.")
+                raise Exception(
+                    f"client:{self.__client_id} had shutdown, can't startup again."
+                )
 
             try:
                 # pre update topic route for producer or consumer
@@ -76,7 +82,8 @@ class Client:
             except Exception as e:
                 # ignore this exception and retrieve again when calling send or receive
                 logger.warn(
-                    f"update topic exception when client startup, ignore it, try it again in scheduler. exception: {e}")
+                    f"update topic exception when client startup, ignore it, try it again in scheduler. exception: {e}"
+                )
             self.__start_scheduler()
             self.__start_async_rpc_callback_handler()
             self.__is_running = True
@@ -110,27 +117,27 @@ class Client:
     """ abstract """
 
     def _start_success(self):
-        """ each subclass implements its own actions after a successful startup """
+        """each subclass implements its own actions after a successful startup"""
         pass
 
     def _start_failure(self):
-        """ each subclass implements its own actions after a startup failure """
+        """each subclass implements its own actions after a startup failure"""
         pass
 
     def _sync_setting_req(self, endpoints):
-        """ each subclass implements its own telemetry settings scheme """
+        """each subclass implements its own telemetry settings scheme"""
         pass
 
     def _heartbeat_req(self):
-        """ each subclass implements its own heartbeat request """
+        """each subclass implements its own heartbeat request"""
         pass
 
     def _notify_client_termination_req(self):
-        """ each subclass implements its own client termination request """
+        """each subclass implements its own client termination request"""
         pass
 
     def _update_queue_selector(self, topic, topic_route):
-        """ each subclass implements its own queue selector """
+        """each subclass implements its own queue selector"""
         pass
 
     """ scheduler """
@@ -140,29 +147,36 @@ class Client:
         try:
             # update topic route every 30 seconds
             self.__client_thread_task_enabled = True
-            self.__topic_route_scheduler = threading.Thread(target=self.__schedule_update_topic_route_cache,
-                                                            name="update_topic_route_schedule_thread")
+            self.__topic_route_scheduler = threading.Thread(
+                target=self.__schedule_update_topic_route_cache,
+                name="update_topic_route_schedule_thread",
+            )
             self.__topic_route_scheduler_threading_event = threading.Event()
             self.__topic_route_scheduler.start()
             logger.info("start topic route scheduler success.")
 
             # send heartbeat to all endpoints every 10 seconds
-            self.__heartbeat_scheduler = threading.Thread(target=self.__schedule_heartbeat,
-                                                          name="heartbeat_schedule_thread")
+            self.__heartbeat_scheduler = threading.Thread(
+                target=self.__schedule_heartbeat, name="heartbeat_schedule_thread"
+            )
             self.__heartbeat_scheduler_threading_event = threading.Event()
             self.__heartbeat_scheduler.start()
             logger.info("start heartbeat scheduler success.")
 
             # send client setting to all endpoints every 5 seconds
-            self.__sync_setting_scheduler = threading.Thread(target=self.__schedule_update_setting,
-                                                             name="sync_setting_schedule_thread")
+            self.__sync_setting_scheduler = threading.Thread(
+                target=self.__schedule_update_setting,
+                name="sync_setting_schedule_thread",
+            )
             self.__sync_setting_scheduler_threading_event = threading.Event()
             self.__sync_setting_scheduler.start()
             logger.info("start sync setting scheduler success.")
 
             # clear unused grpc channel(>30 minutes) every 60 seconds
-            self.__clear_idle_rpc_channels_scheduler = threading.Thread(target=self.__schedule_clear_idle_rpc_channels,
-                                                                        name="clear_idle_rpc_channel_schedule_thread")
+            self.__clear_idle_rpc_channels_scheduler = threading.Thread(
+                target=self.__schedule_clear_idle_rpc_channels,
+                name="clear_idle_rpc_channel_schedule_thread",
+            )
             self.__clear_idle_rpc_channels_threading_event = threading.Event()
             self.__clear_idle_rpc_channels_scheduler.start()
             logger.info("start clear idle rpc channels scheduler success.")
@@ -176,7 +190,7 @@ class Client:
         while True:
             if self.__client_thread_task_enabled is True:
                 self.__topic_route_scheduler_threading_event.wait(30)
-                logger.debug(f"{self.__str__()} run scheduler for update topic route cache.")
+                logger.debug(f"{self.__str__()} run update topic route in scheduler.")
                 # update topic route for each topic in cache
                 topics = self.__topic_route_cache.keys()
                 for topic in topics:
@@ -185,24 +199,29 @@ class Client:
                             self.__update_topic_route_async(topic)
                     except Exception as e:
                         logger.error(
-                            f"{self.__str__()} run scheduler for update topic:{topic} route cache exception: {e}")
+                            f"{self.__str__()} scheduler update topic:{topic} route raise exception: {e}"
+                        )
             else:
                 break
-        logger.info(f"{self.__str__()} stop scheduler for update topic route cache success.")
+        logger.info(
+            f"{self.__str__()} stop scheduler for update topic route cache success."
+        )
 
     def __schedule_heartbeat(self):
         asyncio.set_event_loop(self._rpc_channel_io_loop())
         while True:
             if self.__client_thread_task_enabled is True:
                 self.__heartbeat_scheduler_threading_event.wait(10)
-                logger.debug(f"{self.__str__()} run scheduler for heartbeat.")
+                logger.debug(f"{self.__str__()} run send heartbeat in scheduler.")
                 all_endpoints = self.__get_all_endpoints().values()
                 try:
                     for endpoints in all_endpoints:
                         if self.__client_thread_task_enabled is True:
                             self.__heartbeat_async(endpoints)
                 except Exception as e:
-                    logger.error(f"{self.__str__()} run scheduler for heartbeat exception: {e}")
+                    logger.error(
+                        f"{self.__str__()} scheduler send heartbeat raise exception: {e}"
+                    )
             else:
                 break
         logger.info(f"{self.__str__()} stop scheduler for heartbeat success.")
@@ -212,16 +231,16 @@ class Client:
         while True:
             if self.__client_thread_task_enabled is True:
                 self.__sync_setting_scheduler_threading_event.wait(5)
-                logger.debug(f"{self.__str__()} run scheduler for update setting.")
+                logger.debug(f"{self.__str__()} run update setting in scheduler.")
                 try:
                     all_endpoints = self.__get_all_endpoints().values()
                     for endpoints in all_endpoints:
                         if self.__client_thread_task_enabled is True:
-                            # if stream_stream_call for grpc channel is none, create a new one, otherwise use the existing one
-                            self.__retrieve_telemetry_stream_stream_call(endpoints)
                             self.__setting_write(endpoints)
                 except Exception as e:
-                    logger.error(f"{self.__str__()} run scheduler for update setting exception: {e}")
+                    logger.error(
+                        f"{self.__str__()} scheduler set setting raise exception: {e}"
+                    )
             else:
                 break
         logger.info(f"{self.__str__()} stop scheduler for update setting success.")
@@ -230,15 +249,21 @@ class Client:
         while True:
             if self.__client_thread_task_enabled is True:
                 self.__clear_idle_rpc_channels_threading_event.wait(60)
-                logger.debug(f"{self.__str__()} run scheduler for clear idle rpc channels.")
+                logger.debug(
+                    f"{self.__str__()} run scheduler for clear idle rpc channels."
+                )
                 try:
                     if self.__client_thread_task_enabled is True:
                         self.__rpc_client.clear_idle_rpc_channels()
                 except Exception as e:
-                    logger.error(f"{self.__str__()} run scheduler for clear idle rpc channels: {e}")
+                    logger.error(
+                        f"{self.__str__()} run scheduler for clear idle rpc channels: {e}"
+                    )
             else:
                 break
-        logger.info(f"{self.__str__()} stop scheduler for clear idle rpc channels success.")
+        logger.info(
+            f"{self.__str__()} stop scheduler for clear idle rpc channels success."
+        )
 
     """ callback handler for async method """
 
@@ -247,11 +272,15 @@ class Client:
         # this handler switches user's callback thread from RpcClient's _io_loop_thread to client's callback_handler_thread
         try:
             for i in range(Client.CALLBACK_THREADS_NUM):
-                th = threading.Thread(name=f"callback_handler_thread-{i}", target=self.__handle_callback)
+                th = threading.Thread(
+                    name=f"callback_handler_thread-{i}", target=self.__handle_callback
+                )
                 th.daemon = True
                 self.__callback_threads.append(th)
                 th.start()
-                logger.info(f"{self.__str__()} start async rpc callback thread:{th} success.")
+                logger.info(
+                    f"{self.__str__()} start async rpc callback thread:{th} success."
+                )
         except Exception as e:
             print(f"{self.__str__()} start async rpc callback raise exception: {e}")
             raise e
@@ -260,7 +289,10 @@ class Client:
         while True:
             if self.__client_thread_task_enabled is True:
                 callback_result = self.__callback_result_queue.get()
-                if callback_result.result_type == CallbackResultType.END_CALLBACK_THREAD_RESULT:
+                if (
+                    callback_result.result_type
+                    == CallbackResultType.END_CALLBACK_THREAD_RESULT
+                ):
                     # end infinite loop when client shutdown
                     self.__callback_result_queue.task_done()
                     break
@@ -272,7 +304,9 @@ class Client:
                     self.__callback_result_queue.task_done()
             else:
                 break
-        logger.info(f"{self.__str__()} stop client callback result handler thread:{threading.current_thread()} success.")
+        logger.info(
+            f"{self.__str__()} stop client callback result handler thread:{threading.current_thread()} success."
+        )
 
     """ protect """
 
@@ -282,10 +316,12 @@ class Client:
             return route
         else:
             route = self.__update_topic_route(topic)
-            logger.info(f"{self.__str__()} update topic:{topic} route success.")
             if route is not None:
+                logger.info(f"{self.__str__()} update topic:{topic} route success.")
                 self.__topics.add(topic)
-            return route
+                return route
+            else:
+                raise Exception(f"failed to fetch topic:{topic} route.")
 
     def _remove_unused_topic_route_data(self, topic):
         self.__topic_route_cache.remove(topic)
@@ -306,31 +342,41 @@ class Client:
     # topic route #
 
     def __update_topic_route(self, topic):
-        try:
-            future = self.__rpc_client.query_topic_route_async(self.__client_configuration.rpc_endpoints,
-                                                               self.__topic_route_req(topic), metadata=self._sign(),
-                                                               timeout=self.__client_configuration.request_timeout)
-            res = future.result()
-            route = self.__handle_topic_route_res(res, topic)
-            return route
-        except Exception as e:
-            logger.error(f"update topic route error, topic:{topic}, {e}")
-            raise e
+        event = threading.Event()
+        callback = functools.partial(
+            self.__query_topic_route_async_callback, topic=topic, event=event
+        )
+        future = self.__rpc_client.query_topic_route_async(
+            self.__client_configuration.rpc_endpoints,
+            self.__topic_route_req(topic),
+            metadata=self._sign(),
+            timeout=self.__client_configuration.request_timeout,
+        )
+        future.add_done_callback(callback)
+        event.wait()
+        return self.__topic_route_cache.get(topic)
 
     def __update_topic_route_async(self, topic):
-        callback = functools.partial(self.__query_topic_route_async_callback, topic=topic)
-        future = self.__rpc_client.query_topic_route_async(self.__client_configuration.rpc_endpoints,
-                                                           self.__topic_route_req(topic),
-                                                           metadata=self._sign(),
-                                                           timeout=self.__client_configuration.request_timeout)
+        callback = functools.partial(
+            self.__query_topic_route_async_callback, topic=topic
+        )
+        future = self.__rpc_client.query_topic_route_async(
+            self.__client_configuration.rpc_endpoints,
+            self.__topic_route_req(topic),
+            metadata=self._sign(),
+            timeout=self.__client_configuration.request_timeout,
+        )
         future.add_done_callback(callback)
 
-    def __query_topic_route_async_callback(self, future, topic):
+    def __query_topic_route_async_callback(self, future, topic, event=None):
         try:
             res = future.result()
             self.__handle_topic_route_res(res, topic)
         except Exception as e:
             raise e
+        finally:
+            if event is not None:
+                event.set()
 
     def __topic_route_req(self, topic):
         req = QueryRouteRequest()
@@ -344,7 +390,9 @@ class Client:
             MessagingResultChecker.check(res.status)
             if res.status.code == Code.OK:
                 topic_route = TopicRouteData(res.message_queues)
-                logger.debug(f"{self.__str__()} update topic:{topic} route, route info: {topic_route.__str__()}")
+                logger.info(
+                    f"{self.__str__()} update topic:{topic} route, route info: {topic_route.__str__()}"
+                )
                 # if topic route has new endpoint, connect
                 self.__check_topic_route_endpoints_changed(topic, topic_route)
                 self.__topic_route_cache.put(topic, topic_route)
@@ -359,27 +407,50 @@ class Client:
     def __heartbeat_async(self, endpoints):
         req = self._heartbeat_req()
         callback = functools.partial(self.__heartbeat_callback, endpoints=endpoints)
-        future = self.__rpc_client.heartbeat_async(endpoints, req, metadata=self._sign(),
-                                                   timeout=self.__client_configuration.request_timeout)
+        future = self.__rpc_client.heartbeat_async(
+            endpoints,
+            req,
+            metadata=self._sign(),
+            timeout=self.__client_configuration.request_timeout,
+        )
         future.add_done_callback(callback)
 
     def __heartbeat_callback(self, future, endpoints):
         try:
             res = future.result()
             if res is not None and res.status.code == Code.OK:
-                logger.info(f"{self.__str__()} send heartbeat to {endpoints.__str__()} success.")
+                logger.info(
+                    f"{self.__str__()} send heartbeat to {endpoints.__str__()} success."
+                )
             else:
                 if res is not None:
                     logger.error(
-                        f"{self.__str__()} send heartbeat to {endpoints.__str__()} error, code:{res.status.code}, message:{res.status.message}.")
+                        f"{self.__str__()} send heartbeat to {endpoints.__str__()} error, code:{res.status.code}, message:{res.status.message}."
+                    )
                 else:
-                    logger.error(f"{self.__str__()} send heartbeat to {endpoints.__str__()} error, response is none.")
+                    logger.error(
+                        f"{self.__str__()} send heartbeat to {endpoints.__str__()} error, response is none."
+                    )
         except Exception as e:
-            logger.error(f"{self.__str__()} send heartbeat to {endpoints.__str__()} exception, e: {e}")
+            logger.error(
+                f"{self.__str__()} send heartbeat to {endpoints.__str__()} exception, e: {e}"
+            )
             raise e
 
     # sync settings #
 
+    def __retrieve_telemetry_stream_stream_call(self, endpoints, rebuild=False):
+        try:
+            self.__rpc_client.telemetry_stream(
+                endpoints, self, self._sign(), rebuild, timeout=60 * 60 * 24 * 365
+            )
+        except Exception as e:
+            logger.error(
+                f"{self.__str__()} rebuild stream_steam_call to {endpoints.__str__()} exception: {e}"
+                if rebuild
+                else f"{self.__str__()} create stream_steam_call to {endpoints.__str__()} exception: {e}"
+            )
+
     def __setting_write(self, endpoints):
         req = self._sync_setting_req(endpoints)
         callback = functools.partial(self.__setting_write_callback, endpoints=endpoints)
@@ -387,26 +458,26 @@ class Client:
         logger.debug(f"{self.__str__()} send setting to {endpoints.__str__()}, {req}")
         future.add_done_callback(callback)
 
-    def __retrieve_telemetry_stream_stream_call(self, endpoints, rebuild=False):
-        try:
-            self.__rpc_client.telemetry_stream(endpoints, self, metadata=self._sign(), timeout=60 * 60 * 24 * 365,
-                                               rebuild=rebuild)
-        except Exception as e:
-            logger.error(
-                f"{self.__str__()} rebuild stream_steam_call to {endpoints.__str__()} exception: {e}" if rebuild else f"{self.__str__()} create stream_steam_call to {endpoints.__str__()} exception: {e}")
-
     def __setting_write_callback(self, future, endpoints):
         try:
             future.result()
-            logger.debug(f"{self.__str__()} send setting to {endpoints.__str__()} success.")
+            logger.info(
+                f"{self.__str__()} send setting to {endpoints.__str__()} success."
+            )
         except InvalidStateError as e:
-            logger.warn(f"{self.__str__()} send setting to {endpoints.__str__()} occurred InvalidStateError: {e}")
+            logger.warn(
+                f"{self.__str__()} send setting to {endpoints.__str__()} occurred InvalidStateError: {e}"
+            )
             self.__retrieve_telemetry_stream_stream_call(endpoints, rebuild=True)
         except AioRpcError as e:
-            logger.warn(f"{self.__str__()} send setting to {endpoints.__str__()} occurred AioRpcError: {e}")
+            logger.warn(
+                f"{self.__str__()} send setting to {endpoints.__str__()} occurred AioRpcError: {e}"
+            )
             self.__retrieve_telemetry_stream_stream_call(endpoints, rebuild=True)
         except Exception as e:
-            logger.error(f"{self.__str__()} send setting to {endpoints.__str__()} exception: {e}")
+            logger.error(
+                f"{self.__str__()} send setting to {endpoints.__str__()} exception: {e}"
+            )
             self.__retrieve_telemetry_stream_stream_call(endpoints, rebuild=True)
 
     # metrics #
@@ -418,8 +489,12 @@ class Client:
 
     def __client_termination(self, endpoints):
         req = self._notify_client_termination_req()
-        future = self.__rpc_client.notify_client_termination(endpoints, req, metadata=self._sign(),
-                                                             timeout=self.__client_configuration.request_timeout)
+        future = self.__rpc_client.notify_client_termination(
+            endpoints,
+            req,
+            metadata=self._sign(),
+            timeout=self.__client_configuration.request_timeout,
+        )
         future.result()
 
     # others ##
@@ -435,11 +510,15 @@ class Client:
         old_route = self.__topic_route_cache.get(topic)
         if old_route is None or old_route != route:
             logger.info(
-                f"topic:{topic} route changed for {self.__str__()}. old route is {old_route}, new route is {route}")
+                f"topic:{topic} route changed for {self.__str__()}. old route is {old_route}, new route is {route}"
+            )
         all_endpoints = self.__get_all_endpoints()  # the existing endpoints
-        topic_route_endpoints = route.all_endpoints()  # the latest endpoints for topic route
+        topic_route_endpoints = (
+            route.all_endpoints()
+        )  # the latest endpoints for topic route
         diff = set(topic_route_endpoints.keys()).difference(
-            set(all_endpoints.keys()))  # the diff between existing and latest
+            set(all_endpoints.keys())
+        )  # the diff between existing and latest
         # create grpc channel, stream_stream_call for new endpoints, send setting to new endpoints
         for address in diff:
             endpoints = topic_route_endpoints[address]
@@ -477,7 +556,9 @@ class Client:
                 self.__clear_idle_rpc_channels_scheduler.join()
 
         for i in range(Client.CALLBACK_THREADS_NUM):
-            self._set_future_callback_result(CallbackResult.end_callback_thread_result())
+            self._set_future_callback_result(
+                CallbackResult.end_callback_thread_result()
+            )
 
         for i in range(Client.CALLBACK_THREADS_NUM):
             self.__callback_threads[i].join()
